<?php
/**
 * @author John "ChaosTangent" Noel <me@chaostangent.com>
 * @license http://creativecommons.org/licenses/by/2.0/ Creative Commons Attribution 2.0 Generic
 * @version 1.1
 * @copyright Copyright 2008 John "ChaosTangent" Noel <http://blog.chaostangent.com>
 *
 * Use and modify in any way you see fit just let me know if you do anything
 * cool with it or release it in some way.
 */

define('N', 				4096);
define('F', 				18);
define('THRESHOLD',	2);

if(($argc >= 2) && (file_exists($argv[1])))
{
	if($fh = fopen($argv[1], 'rb'))
	{
		// check first 8 bytes for signature
		if(fread($fh, 8) == 'yanepkDx')
		{
			$hea = unpack('V', fread($fh, 4));
			$headerEntries = $hea[1];

			// get header blocks
			$blocks = array();
			for($i = 0; $i < $headerEntries; $i++)
			{
				$filename = trim(fread($fh, 256));

				$byteOffsetA = unpack('V', fread($fh, 4));
				$unpackedSizeA = unpack('V', fread($fh, 4));
				$packedSizeA = unpack('V', fread($fh, 4));

				$blocks[$i][0] = $filename;
				$blocks[$i][1] = $byteOffsetA[1];
				$blocks[$i][2] = $unpackedSizeA[1];
				$blocks[$i][3] = $packedSizeA[1];
			}

			// create output directory
			$outputDir = ($argc == 3) ? $argv[2] : dirname(__FILE__).DIRECTORY_SEPARATOR.'output'.DIRECTORY_SEPARATOR;
			if(!file_exists($outputDir))
			{
				mkdir($outputDir, 0777, true);
			}

			// output files
			foreach($blocks AS $block)
			{
				$dirname = dirname($block[0]);
				if(!file_exists($outputDir.$dirname))
				{
					mkdir($outputDir.$dirname, 0777, true);
				}

				fseek($fh, $block[1], SEEK_SET);
				$outputFilePath  = $outputDir.$block[0];
				$outputFilePath .= ($block[2] != $block[3]) ? '.lzss' : '';

				if(file_put_contents($outputFilePath, fread($fh, $block[3])))
				{
					echo "Wrote \"{$block[0]}\" ({$block[3]} / {$block[2]})";

					// decompress
					if(($block[2] != $block[3]) &&
						($fi = fopen($outputFilePath, 'rb')) &&
						($fo = fopen($outputDir.$block[0], 'wb')))
					{
						$buffer = array_fill(0, (N + F - 1), ' ');	// N + F - 1
						$c = $i = $j = 0;

						$flags = 0;
						$r = (N - F);

						while(true)
						{
							if((($flags >>= 1) & 0x100) == 0)
							{
								if(($c = fgetc($fi)) === FALSE) break;
								$flags = intval(ord($c)) | 0xFF00;
							}

							if($flags & 1)
							{
								if(($c = fgetc($fi)) === FALSE) break;
								fwrite($fo, $c);
								$buffer[$r++] = $c;
								$r &= (N - 1);
							}
							else
							{
								if(($i = fgetc($fi)) === FALSE) break;
								if(($j = fgetc($fi)) === FALSE) break;

								$i = intval(ord($i)); $j = intval(ord($j));

								$i |= (($j & 0xF0) << 4);
								$j  = ($j & 0x0F) + THRESHOLD;

								for($k = 0; $k <= $j; $k++)
								{
									$c = $buffer[($i + $k) & (N - 1)];
									fwrite($fo, $c);
									$buffer[$r++] = $c;
									$r &= (N - 1);
								}
							}
						}
						fclose($fi);

						fflush($fo);
						fclose($fo);

						unlink($outputFilePath);
						if(filesize($outputDir.$block[0]) == $block[2])
						{
							echo " [decompressed]";
						}
						else
						{
							echo " [corrupt]";
						}
					}
					echo "\n";
				}
			}
		}
		else
		{
			fclose($fh);
			die('File signature does not match expected');
		}

		fclose($fh);
	}
	else
	{
		die('Couldn\'t open file');
	}
}
else
{
	die('Please supply a valid filename');
}

?>